/* Ed25519 command line utilities created by Shaos <me@shaos.net> (20-SEP-2020)

   gcc ed25519.c fe25519.c ge25519.c sc25519.c open.c sign.c sha512.c -o ed25519

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ed25519.h"

int hex2i(char c)
{
  int r;
  switch(c)
  {
    case '0': r = 0; break;
    case '1': r = 1; break;
    case '2': r = 2; break;
    case '3': r = 3; break;
    case '4': r = 4; break;
    case '5': r = 5; break;
    case '6': r = 6; break;
    case '7': r = 7; break;
    case '8': r = 8; break;
    case '9': r = 9; break;
    case 'a': r = 10; break;
    case 'b': r = 11; break;
    case 'c': r = 12; break;
    case 'd': r = 13; break;
    case 'e': r = 14; break;
    case 'f': r = 15; break;
    case 'A': r = 10; break;
    case 'B': r = 11; break;
    case 'C': r = 12; break;
    case 'D': r = 13; break;
    case 'E': r = 14; break;
    case 'F': r = 15; break;
    default:  r = -1;
  }
  return r;
}

int main(int argc, char** argv)
{
 FILE *f;
 int i,s,ok=1;
 unsigned int u;
 unsigned long long len;
 unsigned char *buf,*sbuf;
 unsigned char sk[ED25519_SECRETKEYBYTES]; // or signature
 unsigned char pk[ED25519_PUBLICKEYBYTES];
 if(argc<3)
 {
    printf("\nUsage:\n");
    printf("\ted25519 filename secretkey\n");
    printf("or\n");
    printf("\ted25519 filename signature publickey\n");
    return -1;
 }
 if(strlen(argv[2])==ED25519_SECRETKEYBYTES*2)
 {
  for(i=0;i<ED25519_SECRETKEYBYTES;i++)
  {
    if(!argv[2][i+i]) return -11;
    s = hex2i(argv[2][i+i]);
    if(s < 0) return -12;
    sk[i] = s<<4;
    if(!argv[2][i+i+1]) return -13;
    s = hex2i(argv[2][i+i+1]);
    if(s < 0) return -14;
    sk[i] |= s;
  }
 }
 else // is it a filename?
 {
  f = fopen(argv[2],"r");
  if(f==NULL) return -10;
  printf("Open file '%s'\n",argv[2]);
  fseek(f,0,SEEK_END);
  s = ftell(f);
  fseek(f,0,SEEK_SET);
  if(s!=ED25519_SECRETKEYBYTES) return -20;
  for(i=0;i<ED25519_SECRETKEYBYTES;i++) sk[i]=fgetc(f);
  fclose(f);
 }
 if(argc>3)
 {
  for(i=0;i<ED25519_PUBLICKEYBYTES;i++)
  {
    if(!argv[3][i+i]) return -21;
    s = hex2i(argv[3][i+i]);
    if(s < 0) return -22;
    pk[i] = s<<4;
    if(!argv[3][i+i+1]) return -23;
    s = hex2i(argv[3][i+i+1]);
    if(s < 0) return -24;
    pk[i] |= s;
  }
 }
 f = fopen(argv[1],"r");
 if(f==NULL) return -2;
 fseek(f,0,SEEK_END);
 s = ftell(f);
 fseek(f,0,SEEK_SET);
 buf = (unsigned char*)malloc(s);
 if(buf==NULL)
 {
    fclose(f);
    return -3;
 }
 sbuf = (unsigned char*)malloc(s+ED25519_SIGNATUREBYTES);
 if(buf==NULL)
 {
    free(buf);
    fclose(f);
    return -4;
 }
 for(i=0;i<s;i++) buf[i]=fgetc(f);
 fclose(f);
 if(argc==3)
 {
  crypto_sign_ed25519(sbuf,&len,buf,s,sk);
  printf("Ed25519 signature for %s:\n",argv[1]);
  for(i=0;i<64;i++)
  {
    printf("%2.2x",sbuf[i]);
  }
  printf("\n");
  u = sbuf[0]<<24;
  u |= sbuf[1]<<16;
  u |= sbuf[2]<<8;
  u |= sbuf[3];
#if 1
  printf("\n");
//  printf("%8.8X -> %1.10f\n",1,((double)1)/4294967296.0);
//  printf("%8.8X -> %1.10f\n",2,((double)2)/4294967296.0);
  printf("%8.8X -> %1.10f\n",u,((double)u)/4294967296.0);
#endif
 }
 else
 {
  memcpy(sbuf,sk,ED25519_SIGNATUREBYTES);
  memcpy(sbuf+ED25519_SIGNATUREBYTES,buf,s);
  if(!crypto_sign_open_nocopy_ed25519(sbuf,s+ED25519_SIGNATUREBYTES,pk))
  {
     printf("Valid\n");
  }
  else
  {
     printf("Invalid\n");
     ok = 0;
  }
 }
 free(buf);
 free(sbuf);
 if(ok) return 0;
 else return -100;
}
