/* Public domain created by
   Daniel J. Bernstein
   Niels Duif
   Tanja Lange
   lead: Peter Schwabe
   Bo-Yin Yang
*/

#define ED25519_SECRETKEYBYTES 64
#define ED25519_PUBLICKEYBYTES 32
#define ED25519_SIGNATUREBYTES 64

/* The crypto_sign_keypair() function randomly generates a secret key and
   a corresponding public key. The public key is put into
   pk (crypto_sign_PUBLICKEYBYTES bytes) and the secret key into
   sk (crypto_sign_SECRETKEYBYTES bytes).
*/

int crypto_sign_keypair_ed25519(unsigned char *pk,unsigned char *sk);

/* The crypto_sign() function prepends a signature to a message m whose length
   is mlen bytes, using the secret key sk. The signed message, which includes
   the signature + a plain copy of the message, is put into sm, and is
   crypto_sign_BYTES + mlen bytes long. If smlen is not a NULL pointer,
   the actual length of the signed message is stored into smlen.
*/

int crypto_sign_ed25519(
    unsigned char *sm,unsigned long long *smlen,
    const unsigned char *m,unsigned long long mlen,
    const unsigned char *sk
    );

/* The crypto_sign_open() function checks that the signed message sm whose
   length is smlen bytes has a valid signature for the public key pk.
   If the signature is doesn't appear to be valid, the function returns -1.
   On success, it puts the message with the signature removed into m,
   stores its length into mlen if mlen is not a NULL pointer, and returns 0.
*/

int crypto_sign_open_ed25519(
    unsigned char *m,unsigned long long *mlen,
    /*const*/ unsigned char *sm,unsigned long long smlen,
    const unsigned char *pk
    );

/* Here sm is 64-byte signature and then data, so smlen must be data size plus 64 */

#define crypto_sign_open_nocopy_ed25519(sm,smlen,pk) crypto_sign_open_ed25519(NULL,NULL,sm,smlen,pk)
