#include <string.h>
//#include "crypto_sign.h"
#include "crypto_hash_sha512.h"
#include "randombytes.h"
#include "ge25519.h"

/* The crypto_sign_keypair() function randomly generates a secret key and
   a corresponding public key. The public key is put into
   pk (crypto_sign_PUBLICKEYBYTES bytes) and the secret key into
   sk (crypto_sign_SECRETKEYBYTES bytes).
*/

int crypto_sign_keypair_ed25519(unsigned char *pk,unsigned char *sk)
{
  unsigned char az[64];
  sc25519 scsk;
  ge25519 gepk;

  randombytes(sk,32);
  crypto_hash_sha512(az,sk,32);
  az[0] &= 248;
  az[31] &= 127;
  az[31] |= 64;

  sc25519_from32bytes(&scsk,az);

  ge25519_scalarmult_base(&gepk, &scsk);
  ge25519_pack(pk, &gepk);
  memmove(sk + 32,pk,32);
  return 0;
}

#ifdef MAIN
#include <stdio.h>
int main()
{
  int i;
  unsigned char pk[32],sk[64];
  crypto_sign_keypair_ed25519(pk,sk);
  printf("\nEd25519 secret key:\n");
  for(i=0;i<64;i++) printf("%2.2x",sk[i]);
  printf("\n\nEd25519 public key:\n");
  for(i=0;i<32;i++) printf("%2.2x",pk[i]);
  printf("\n\n");
  return 0;
}
#endif
