/* Ed25519 test by Shaos <me@shaos.net> (20-SEP-2020)

   gcc test.c fe25519.c ge25519.c sc25519.c open.c sign.c sha512.c -o ed25519test

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "ed25519.h"

int hex2i(char c)
{
  int r;
  switch(c)
  {
    case '0': r = 0; break;
    case '1': r = 1; break;
    case '2': r = 2; break;
    case '3': r = 3; break;
    case '4': r = 4; break;
    case '5': r = 5; break;
    case '6': r = 6; break;
    case '7': r = 7; break;
    case '8': r = 8; break;
    case '9': r = 9; break;
    case 'a': r = 10; break;
    case 'b': r = 11; break;
    case 'c': r = 12; break;
    case 'd': r = 13; break;
    case 'e': r = 14; break;
    case 'f': r = 15; break;
    case 'A': r = 10; break;
    case 'B': r = 11; break;
    case 'C': r = 12; break;
    case 'D': r = 13; break;
    case 'E': r = 14; break;
    case 'F': r = 15; break;
    default:  r = -1;
  }
  return r;
}

int main()
{
 FILE *f;
 time_t t1,t2;
 int i,j,s,skip,ok=1;
 unsigned int u;
 unsigned long long len;
 unsigned char *buf,*sbuf;
 double sum = 0.0;
 char str[5000],*po,zero;
 unsigned char sk[ED25519_SECRETKEYBYTES]; // or signature
 unsigned char pk[ED25519_PUBLICKEYBYTES];
 f = fopen("sign.input","r");
 if(f==NULL) return -1;
 j = 0;
 t1 = time(NULL);
 while(ok)
 {
    j++;
    fgets(str,5000,f);
    if(feof(f)) break;
    po = strtok(str,":");
    if(po==NULL) return -2;
    for(i=0;i<ED25519_SECRETKEYBYTES;i++)
    {
      if(!po[i+i]) return -11;
      s = hex2i(po[i+i]);
      if(s < 0) return -12;
      sk[i] = s<<4;
      if(!po[i+i+1]) return -13;
      s = hex2i(po[i+i+1]);
      if(s < 0) return -14;
      sk[i] |= s;
    }
    po = strtok(NULL,":");
    if(po==NULL) return -3;
    if(po[strlen(po)+1]==':')
         skip = 1;
    else skip = 0;
    for(i=0;i<ED25519_PUBLICKEYBYTES;i++)
    {
      if(!po[i+i]) return -21;
      s = hex2i(po[i+i]);
      if(s < 0) return -22;
      pk[i] = s<<4;
      if(!po[i+i+1]) return -23;
      s = hex2i(po[i+i+1]);
      if(s < 0) return -24;
      pk[i] |= s;
    }
    if(skip) po = &zero; // workaround for strtok issue with empty columns
    else po = strtok(NULL,":");
    if(po==NULL) return -4;
    buf = (unsigned char*)malloc(strlen(po)/2+1);
    sbuf = (unsigned char*)malloc(strlen(po)/2+ED25519_SIGNATUREBYTES);
    for(i=0;i<strlen(po)/2;i++)
    {
      if(!po[i+i]) return -31;
      s = hex2i(po[i+i]);
      if(s < 0) return -32;
      buf[i] = s<<4;
      if(!po[i+i+1]) return -33;
      s = hex2i(po[i+i+1]);
      if(s < 0) return -34;
      buf[i] |= s;
    }
    po = strtok(NULL,":");
    if(po==NULL) return -5;
    s = i;
    crypto_sign_ed25519(sbuf,&len,buf,s,sk);
    printf("Ed25519 signature for %i (%i->%i):",j,s,len);
    for(i=0;i<64;i++)
    {
      printf("%2.2x",sbuf[i]);
    }
    u = sbuf[0]<<24;
    u |= sbuf[1]<<16;
    u |= sbuf[2]<<8;
    u |= sbuf[3];
    sum += ((double)u)/4294967296.0;
#if 1
    printf(" | %8.8X -> %1.10f ",u,((double)u)/4294967296.0);
#endif
    for(i=0;i<strlen(po)/2;i++)
    {
      if(!po[i+i]) return -41;
      s = hex2i(po[i+i]);
      if(s < 0) return -42;
      if((sbuf[i]>>4) != s)
      {
         printf(" ERR1");
         break;
      }
      if(!po[i+i+1]) return -43;
      s = hex2i(po[i+i+1]);
      if(s < 0) return -44;
      if((sbuf[i]&15) != s)
      {
         printf(" ERR2");
         break;
      }
    }
    if(!crypto_sign_open_nocopy_ed25519(sbuf,len,pk))
    {
     printf(" Valid\n");
    }
    else
    {
     printf(" Invalid\n");
     ok = 0;
    }
    free(buf);
    free(sbuf);
 }
 t2 = time(NULL);
 printf("Ok=%i dT=%i avg=%1.10f\n",ok,(int)(t2-t1),sum/j);
 if(ok) return 0;
 else return -100;
}
